from .xmi_enums import XmiEnum
from enum import unique


@unique
class XmiStructuralCurveMemberTypeEnum(XmiEnum):
    BEAM = (1, "Beam")
    COLUMN = (2, "Column")
    BRACING = (3, "Bracing")
    OTHER = (4, "Other")


@unique
class XmiStructuralCurveMemberSystemLineEnum(XmiEnum):
    TOP_MIDDLE = (1, "TopMiddle")
    TOP_LEFT = (2, "TopLeft")
    TOP_RIGHT = (3, "TopRight")
    MIDDLE_MIDDLE = (4, "MiddleMiddle")
    MIDDLE_LEFT = (5, "MiddleLeft")
    MIDDLE_RIGHT = (6, "MiddleRight")
    BOTTOM_LEFT = (7, "BottomLeft")
    BOTTOM_MIDDLE = (8, "BottomMiddle")
    BOTTOM_RIGHT = (9, "BottomRight")



from .xmi_enums import XmiEnum
from enum import unique


@unique
class XmiStructuralMaterialTypeEnum(XmiEnum):
    CONCRETE = (1, "Concrete")
    STEEL = (2, "Steel")
    TIMBER = (3, "Timber")
    ALUMINIUM = (4, "Aluminium")
    COMPOSITE = (5, "Composite")
    MASONRY = (6, "Masonry")
    OTHERS = (7, "Others")
    REBAR = (8, "Rebar")  # to be removed
    TENDON = (9, "Tendon")  # to be removed



from .xmi_has_structural_material import XmiHasStructuralMaterial
from .xmi_has_structural_node import XmiHasStructuralNode
from .xmi_has_structural_cross_section import XmiHasStructuralCrossSection
from .xmi_has_point_3d import XmiHasPoint3D
from .xmi_has_segment import XmiHasSegment
from .xmi_has_geometry import XmiHasGeometry



from .xmi_enums import XmiEnum
from enum import unique


@unique
class XmiStructuralSurfaceMemberTypeEnum(XmiEnum):
    SLAB = (1, "Slab")
    WALL = (2, "Wall")
    PAD_FOOTING = (3, "PadFooting")
    STRIP_FOOTING = (4, "StripFooting")
    PILECAP = (5, "Pilecap")  # i think this shouldn't be here
    ROOF_PANEL = (6, "RoofPanel")  # this sounds like archi. should not be here
    WALL_PANEL = (7, "WallPanel")  # this sounds like archi. should not be here
    # For future changes to be shifted to foundation type elements
    RAFT = (8, "Raft")


@unique
class XmiStructuralSurfaceMemberSpanTypeEnum(XmiEnum):
    ONE_WAY = (1, "OneWay")
    TWO_WAY = (2, "TwoWay")


@unique
class XmiStructuralSurfaceMemberSystemPlaneEnum(XmiEnum):
    BOTTOM = (1, "Bottom")
    TOP = (2, "Top")
    MIDDLE = (3, "Middle")
    LEFT = (4, "Left")
    RIGHT = (5, "Right")



from .xmi_structural_cross_section import XmiStructuralCrossSection
from .xmi_structural_curve_member import XmiStructuralCurveMember
from .xmi_structural_material import XmiStructuralMaterial
from .xmi_structural_point_connection import XmiStructuralPointConnection
from .xmi_structural_surface_member import XmiStructuralSurfaceMember
from .xmi_structural_unit import XmiStructuralUnit
from .xmi_segment import XmiSegment



from enum import Enum, unique
from ..geometries.xmi_line_3d import XmiLine3D
from ..geometries.xmi_arc_3d import XmiArc3D


@unique
class XmiEnum(Enum):

    @classmethod
    def from_name_get_enum(cls, name_str):
        try:
            return cls[name_str]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    @classmethod
    def from_attribute_get_enum(cls, attribute_str: str):
        for member in cls:
            if member.value[1] == attribute_str:
                return member
        return None  # Or raise a custom exception if you prefer

    @classmethod
    def from_name_get_enum_attribute(cls, name_str: str):
        try:
            return cls[name_str].value[1]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    def get_attribute_value(cls):
        try:
            return cls.value[1]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    @classmethod
    def get_attribute_value(cls, enum: Enum):
        try:
            return enum.value[1]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    @classmethod
    def from_name_get_enum_value(cls, name_str: str):
        try:
            return cls[name_str].value[0]
        except KeyError:
            return None  # Or raise a custom exception if you prefer


@unique
class XmiUnitEnum(XmiEnum):
    METER3 = (1, "m^3")
    METER2 = (2, "m^2")
    METER = (3, "m")
    METER4 = (4, "m^4")
    MILLIMETER4 = (5, "mm^4")
    MILLIMETER = (6, "mm")
    CENTIMETER = (7, "cm")
    MILLIMETER3 = (8, "mm^3")
    MILLIMETER2 = (9, "mm^2")
    SECOND = (10, "sec")


@unique
class XmiSegmentTypeEnum(XmiEnum):
    LINE = (1, "Line", XmiLine3D)
    CIRCULAR_ARC = (2, "Circular Arc", XmiArc3D)
    PARABOLIC_ARC = (3, "Parabolic Arc")
    BEZIER = (4, "Bezier")
    SPLINE = (5, "Spline")
    OTHERS = (6, "Others")

    def get_geometry_class(self):
        try:
            if len(self.value) < 3:
                return None
            return self.value[2]
        except KeyError:
            return None  # Or raise a custom exception if you prefer



# Optional, for forward declarations in Python 3.7+
from __future__ import annotations

from .xmi_enums import XmiEnum
from enum import unique, Enum

# After adding these shapes, an image of how it is being defined is needed


@unique
class XmiStructuralCrossSectionShapeEnum(Enum):
    RECTANGULAR = (1, "Rectangular", 2, ("H", "B"))
    CIRCULAR = (2, "Circular", 1, ("D"))
    L_SHAPE = (3, "L Shape", 4, ("H", "B", "T", "t"))
    T_SHAPE = (4, "T Shape", 4, ("H", "B", "T", "t"))
    C_SHAPE = (5, "C Shape", 5, ("H", "B", "T1", "T2", "t"))
    I_SHAPE = (6, "I Shape", 5, ("D", "B", "T", "t", "r"))
    SQUARE_HOLLOW = (7, "Square Hollow", 2, ("D", "t"))
    RECTANGULAR_HOLLOW = (8, "Rectangular Hollow", 3, ("D", "B", "t"))
    OTHERS = (9, "Others", 0, ())

    @classmethod
    def from_name_get_enum(cls, name_str):
        try:
            return cls[name_str]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    @classmethod
    def from_attribute_get_enum(cls, attribute_str: str):
        for member in cls:
            if member.value[1] == attribute_str:
                return member
        return None  # Or raise a custom exception if you prefer

    @classmethod
    def from_name_get_enum_attribute(cls, name_str: str):
        try:
            return cls[name_str].value[1]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    @classmethod
    def get_attribute(cls):
        try:
            return cls.value[1]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    @classmethod
    def from_name_get_enum_value(cls, name_str: str):
        try:
            return cls[name_str].value[0]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    @classmethod
    def get_quantity_of_cross_section_params(cls, enum: Enum):
        try:
            return enum.value[2]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    def get_quantity_of_cross_section_params(self):
        try:
            return self.value[2]
        except KeyError:
            return None  # Or raise a custom exception if you prefer

    @classmethod
    def get_cross_section_params(cls):
        try:
            return cls.value[3]
        except KeyError:
            return None  # Or raise a custom exception if you prefer
